import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;

import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * Die Klasse Koordinatensystem erlaubt, einzelne Punkte in ein Koordinatensystem
 * einzutragen. Die Achsen werden automatisch gezeichnet. Hintergrund-, Achsen-
 * und Zeichenfarbe knnen aus einer kleinen Palette von Farbwrtern ausgewhlt
 * werden. In der gegenwrtigen Fassung sind die Mae des Koordinatensystems noch
 * fix.
 * 
 * Basierend auf dem Projekt Figuren.
 * 
 * @author: Bruce Quig
 * @author: Michael Klling (mik)
 * @author: Axel Schmolitzky
 * 
 * Umgeschrieben als Koordinatensystem.
 * 
 * @author: Bernd Gramlich
 * 
 * @version: 2008-12-16 (leichte Veraenderung 2013-11-25)
 */
public class Koordinatensystem {

    private static Koordinatensystem koordinatensystemSingleton;

    /**
     * Liefert das einzige Exemplar des Koordinatensystems zurck.
     */
    public static Koordinatensystem gibKoordinatensystem() {
        if (koordinatensystemSingleton == null) {
            koordinatensystemSingleton = new Koordinatensystem(
            "Koordinatensystem in BlueJ", 500, 500, "weiss", "schwarz");
        }
        koordinatensystemSingleton.setzeSichtbarkeit(true);
        return koordinatensystemSingleton;
    }

    private JFrame fenster;

    private Zeichenflaeche zeichenflaeche;

    private Graphics2D graphic;

    private String hintergrundfarbe;
    
    private String achsenfarbe;

    private Image leinwandImage;
    /**
     * Konstruktor.
     * Erzeugt ein Koordinatensystem und zeichnet die Koordinatenachsen.
     * 
     * @param titel
     *            Titel, der im Rahmen des Koordinatensystems angezeigt wird.
     * @param breite
     *            die gewnschte Breite des Koordinatensystems
     * @param hoehe
     *            die gewnschte Hhe des Koordinatensystems
     * @param grundfarbe
     *            die Hintergrundfarbe des Koordinatensystems
     * @param strichfarbe
     *            die Achsenfarbe des Koordinatensystems          
     */
    private Koordinatensystem(String titel, int breite, int hoehe, String grundfarbe, String strichfarbe)
    {
        fenster = new JFrame();
        zeichenflaeche = new Zeichenflaeche ();
        fenster.setContentPane (zeichenflaeche);
        fenster.setTitle (titel);
        zeichenflaeche.setPreferredSize (new Dimension (breite, hoehe));
        hintergrundfarbe = grundfarbe;
        achsenfarbe = strichfarbe;
        fenster.pack ();
        setzeSichtbarkeit (true);
        achsen ();
    }

    /**
     * Mache Koordinatensystem sichtbar oder unsichtbar.
     * 
     * @param sichtbar
     *            true fr sichtbar,
     *            false fr nicht sichtbar.
     */
    private void setzeSichtbarkeit(boolean sichtbar)
    {
        if (graphic == null)
        {
            // erstmaliger Aufruf: erzeuge das Bildschirm-Image und flle
            // es mit der Hintergrundfarbe
            Dimension size = zeichenflaeche.getSize ();
            leinwandImage = zeichenflaeche.createImage (size.width, size.height);
            graphic = (Graphics2D) leinwandImage.getGraphics ();
            setzeZeichenfarbe (hintergrundfarbe);
            graphic.fillRect (0, 0, size.width, size.height);
            setzeZeichenfarbe (achsenfarbe);
        }
        fenster.setVisible (sichtbar);
    }

    /**
     * Zeichne den Punkt mit Leinwandkoordinaten (x|y) in der aktuellen Farbe.
     * 
     * @param x
     *            x-Koordinate auf der Leinwand
     * @param y
     *            y-Koordinate auf der Leinwand
     */
    private void zeichne (int x, int y)
    {
        graphic.fill (new Rectangle (x, y, 1, 1));
        zeichenflaeche.repaint ();
    }
    
    /**
     * Zeichne die Koordinatenachsen.
     */
    private void achsen ()
    {
        int n;
        double i;
        
        /**
         * Achsen.
         */
        for (i = -10; i <= 10; i += .04)
        {
            zeichnePunkt (i, 0, achsenfarbe);
            zeichnePunkt (0, i, achsenfarbe);
        }
        
        /**
         * Unterteilungsstriche.
         */
        for (n = -9; n <= 9; n++)
        {
            for (i = 0; i >= -0.2; i -= .04)
            {
                zeichnePunkt (i, n, achsenfarbe);
                zeichnePunkt (n, i, achsenfarbe);
            }
        }
    }
    
    /**
     * Zeichne den Punkt mit mathematischen Koordinaten (x|y) in der
     * gewnschten Farbe.
     * 
     * @param x
     *            x-Koordinate
     * @param y
     *            y-Koordinate
     * @param farbe
     *            gewnschte Farbe
     */
    public void zeichnePunkt (double x, double y, String farbe)
    {
        setzeZeichenfarbe (farbe);
        zeichne (xLeinwand (x), yLeinwand (y));        
    }
    
    /**
     * Berechne aus der mathematischen Koordinate x die
     * Leinwandkoordinate.
     */
    private int xLeinwand (double x)
    {
        return (int) Math.round (x * 25) + 250;
    }
    
    /**
     * Berechne aus der mathematischen Koordinate y die
     * Leinwandkoordinate.
     */    
    private int yLeinwand (double y)
    {
        return 250 - (int) Math.round (y * 25);
    }

    /**
     * Lsche den Punkt mit mathematischen Koordinaten (x|y).
     * 
     * @param x
     *            x-Koordinate
     * @param y
     *            y-Koordinate
     */
    public void loeschePunkt (double x, double y)
    {
        /**
         * Wenn der Punkt ein Achsen- oder Unteilungspunkt ist, frbe ihn
         * in Achsenfarbe ...
         */
        if ((xLeinwand (x) == xLeinwand (0)) || (yLeinwand (y) == yLeinwand (0))
        || (xLeinwand (x) == xLeinwand (Math.round (x))) && (yLeinwand (y) <= yLeinwand (-0.2)) && (yLeinwand (y) >= yLeinwand (0))
        || (yLeinwand (y) == yLeinwand (Math.round (y))) && (xLeinwand (x) >= xLeinwand (-0.2)) && (xLeinwand (x) <= xLeinwand (0)))
        {
            setzeZeichenfarbe (achsenfarbe);
        }
        /**
         * ... sonst frbe ihn in Hintergrundfarbe.
         */
        else
        {
            setzeZeichenfarbe (hintergrundfarbe);
        }
        
        zeichne (xLeinwand (x), yLeinwand (y));
    }
        
    /**
     * Setze die Zeichenfarbe des Koordinatensystems.
     * 
     * @param farbe
     *            der Name der neuen Zeichenfarbe.
     */
    private void setzeZeichenfarbe(String farbe) {
        if (farbe.equals("rot")) {
            graphic.setColor(Color.red);
        } else if (farbe.equals("schwarz")) {
            graphic.setColor(Color.black);
        } else if (farbe.equals("blau")) {
            graphic.setColor(Color.blue);
        } else if (farbe.equals("gelb")) {
            graphic.setColor(Color.yellow);
        } else if (farbe.equals("gruen")) {
            graphic.setColor(Color.green);
        } else if (farbe.equals("lila")) {
            graphic.setColor(Color.magenta);
        } else if (farbe.equals("weiss")) {
            graphic.setColor(Color.white);
        } else if (farbe.equals("grau")) {
            graphic.setColor(Color.gray);
        } else if (farbe.equals("orange")) {
            graphic.setColor(Color.orange);
        } else if (farbe.equals("tuerkis")) {
            graphic.setColor(Color.cyan);
        } else if (farbe.equals("pink")) {
            graphic.setColor(Color.pink);
        } else {
            graphic.setColor(Color.black);
        }
    }

    /**
     * Warte fr die angegebenen Millisekunden. Mit dieser Operation wird eine
     * Verzgerung definiert, die fr animierte Zeichnungen benutzt werden kann.
     * 
     * @param millisekunden
     *            die zu wartenden Millisekunden
     */
    public void warte(int millisekunden) {
        try {
            Thread.sleep(millisekunden);
        } catch (Exception e) {
            // Exception ignorieren
        }
    }

    /**
     * Lsche das gesamte Koordinatensystem.
     */
    private void loeschen() {
        Color original = graphic.getColor();
        setzeZeichenfarbe (hintergrundfarbe);
        Dimension size = zeichenflaeche.getSize();
        graphic.fill(new Rectangle(0, 0, size.width, size.height));
        achsen ();
        graphic.setColor (original);
    }

    /***************************************************************************
     * Interne Klasse Zeichenflaeche - die Klasse fr die GUI-Komponente, die
     * tatschlich im Leinwand-Fenster angezeigt wird. Diese Klasse definiert
     * ein JPanel mit der zustzlichen Mglichkeit, das auf ihm gezeichnet Image
     * aufzufrischen (erneut zu zeichnen).
     */
    private class Zeichenflaeche extends JPanel {
        private static final long serialVersionUID = 20060330L;

        public void paint(Graphics g) {
            g.drawImage(leinwandImage, 0, 0, null);
        }
    }
}
